@@ -356,8 +356,8 @@ impl OptChaining {
         let span = e.span;
         let cons = undefined(span);
 
-        match &mut *e.expr {
-            Expr::Member(MemberExpr {
+        match &mut e.base {
+            OptChainBase::Member(MemberExpr {
                 obj,
                 prop,
                 span: m_span,
@@ -368,22 +368,22 @@ impl OptChaining {
                 let obj_span = obj.span;
                 let obj = self.unwrap(&mut obj);
 
-                let alt = Box::new(Expr::Member(MemberExpr {
+                let alt = OptChainBase::Member(MemberExpr {
                     span: *m_span,
                     obj: obj.alt,
                     prop: prop.take(),
-                }));
+                });
                 let alt = Box::new(Expr::OptChain(OptChainExpr {
                     span: obj_span,
                     question_dot_token,
-                    expr: alt,
+                    base: alt,
                 }));
 
                 return CondExpr { alt, ..obj };
             }
-            Expr::Call(CallExpr {
+            OptChainBase::Call(OptCall {
                 span,
-                callee: Callee::Expr(callee),
+                callee,
                 args,
                 type_args,
             }) if callee.is_opt_chain() => {
@@ -392,16 +392,16 @@ impl OptChaining {
 
                 let obj = self.unwrap(&mut callee);
 
-                let alt = Box::new(Expr::Call(CallExpr {
+                let alt = OptChainBase::Call(OptCall {
                     span: *span,
-                    callee: Callee::Expr(obj.alt),
+                    callee: obj.alt,
                     args: args.take(),
                     type_args: type_args.take(),
-                }));
+                });
                 let alt = Box::new(Expr::OptChain(OptChainExpr {
                     span: *span,
                     question_dot_token,
-                    expr: alt,
+                    base: alt,
                 }));
 
                 return CondExpr {
@@ -413,14 +413,14 @@ impl OptChaining {
             _ => {}
         }
 
-        e.expr.visit_mut_children_with(self);
+        e.base.visit_mut_children_with(self);
 
-        match &mut *e.expr {
-            Expr::Member(MemberExpr { obj, prop, .. }) => {
+        match &mut e.base {
+            OptChainBase::Member(MemberExpr { obj, prop, .. }) => {
                 let obj_span = obj.span();
 
                 let (left, right, alt) = match &mut **obj {
-                    Expr::Ident(..) => (obj.clone(), obj.clone(), e.expr.take()),
+                    Expr::Ident(..) => (obj.clone(), obj.clone(), Box::new(e.base.take().into())),
                     _ => {
                         let i = private_ident!(obj_span, "ref");
                         self.vars_without_init.push(VarDeclarator {
@@ -481,40 +481,38 @@ impl OptChaining {
                 }
             }
 
-            Expr::Call(CallExpr {
-                callee: Callee::Expr(obj),
-                args,
-                type_args,
-                ..
-            }) => {
-                let obj_span = obj.span();
-                let is_super_access = matches!(**obj, Expr::SuperProp(_));
+            OptChainBase::Call(call) => {
+                let callee = &mut call.callee;
+                let obj_span = callee.span();
+                let is_super_access = matches!(&**callee, Expr::SuperProp(_));
 
-                let (left, right, alt) = match &**obj {
+                let (left, right, alt) = match &**callee {
                     Expr::Ident(ident) => (
-                        obj.clone(),
-                        obj.clone(),
+                        callee.clone(),
+                        callee.clone(),
                         if ident.sym == js_word!("eval") {
-                            Box::new(e.expr.take().expect_call().into_indirect().into())
+                            Box::new(CallExpr::from(call.take()).into_indirect().into())
                         } else {
-                            e.expr.take()
+                            Box::new(CallExpr::from(call.take()).into())
                         },
                     ),
-                    _ if is_simple_expr(obj) && self.c.pure_getter => {
-                        (obj.clone(), obj.clone(), e.expr.take())
-                    }
+                    _ if is_simple_expr(callee) && self.c.pure_getter => (
+                        callee.clone(),
+                        callee.clone(),
+                        Box::new(CallExpr::from(call.take()).into()),
+                    ),
                     _ => {
                         let this_as_super;
-                        let should_call = obj.is_member() || obj.is_super_prop();
+                        let should_call = callee.is_member() || callee.is_super_prop();
                         let (this_obj, aliased) = if should_call {
                             alias_if_required(
-                                match &**obj {
+                                match &**callee {
                                     Expr::SuperProp(m) => {
                                         this_as_super = Expr::This(ThisExpr { span: m.obj.span });
                                         &this_as_super
                                     }
                                     Expr::Member(m) => &*m.obj,
-                                    _ => &*obj,
+                                    _ => &*call.callee,
                                 },
                                 "_obj",
                             )
@@ -529,7 +527,7 @@ impl OptChaining {
                                 init: None,
                             });
 
-                            match &mut **obj {
+                            match &mut *call.callee {
                                 Expr::Member(obj) => Box::new(Expr::Member(MemberExpr {
                                     span: obj.span,
                                     obj: Expr::Assign(AssignExpr {
@@ -545,11 +543,11 @@ impl OptChaining {
                                     span: DUMMY_SP,
                                     op: op!("="),
                                     left: PatOrExpr::Pat(this_obj.clone().into()),
-                                    right: obj.take(),
+                                    right: call.callee.take(),
                                 })),
                             }
                         } else {
-                            obj.take()
+                            call.callee.take()
                         };
 
                         let tmp = private_ident!(obj_span, "ref");
@@ -586,12 +584,12 @@ impl OptChaining {
                                     } else {
                                         this_obj.as_arg()
                                     })
-                                    .chain(args.take())
+                                    .chain(call.args.take())
                                     .collect()
                                 } else {
-                                    args.take()
+                                    call.args.take()
                                 },
-                                type_args: type_args.take(),
+                                type_args: call.type_args.take(),
                             })),
                         )
                     }
@@ -630,7 +628,6 @@ impl OptChaining {
                     alt,
                 }
             }
-            _ => unreachable!("TsOptChain.expr = {:?}", e.expr),
         }
     }
 }
